\\ ---------- Methods for the computation of periods ----------

CY_Indicials(L,p)={
    L=polcoef(CY_Pointchange(L,p),0,z);
    local(indicials=factor(L));
    Set(vector(matsize(indicials)[1],X,[-polcoef(indicials[X,1],0,Theta)/polcoef(indicials[X,1],1,Theta),indicials[X,2]]))
};

CY_Pointchange(L,p)={
    local(tmp=1,Lnew=0);
    if(
        p!=oo,for(k=0,4,Lnew+=subst(polcoef(L,k,Theta),z,z+p)*tmp;tmp=(z+p)/z*(z*deriv(tmp,z)+tmp*Theta)),
        p==oo,Lnew=subst(subst(L,Theta,-Theta),z,1/z)
    );
    Lnew/z^(-poldegree(subst(lift(Lnew),z,1/z),z))
};

CY_Solve(L,p,ord,numval)={
    local(ind=CY_Indicials(L,p),indset=Set(apply(x->x[1],ind)));
    L=CY_Pointchange(L,p);
    local(f=vector(#ind,X,vector(ind[X][2],Y,0)));
    local(disc,num,c=0);
    for(X=1,#f,
        f[X][1]=z^ind[X][1]*(1+numval-numval);
        for(k=ind[X][1]+1,ord,
            for(Y=1,ind[X][2],
                disc=subst(polcoef(L,0,z),Theta,k);
                if(setsearch(indset,k)&&Y<ind[X][2],disc=subst(deriv(polcoef(L,0,z),Theta),Theta,k);c=1);
                if(setsearch(indset,k)==0||c==1,
                    num=sum(i=0,min(poldegree(L,z),k),sum(j=1,Y,subst(diffop(polcoef(L,i,z),[Theta],[1],j-1+c),Theta,k-i)/(j-1)!*polcoef(f[X][Y+1-j],k-i,z)));
                    f[X][Y]+=-num/disc*z^k
                );
                c=0;
            )
        )
    );
    f=apply(x->vector(#x,Y,sum(k=0,Y-1,x[Y-k]/k!*Log^k)),f);
    for(i=2,#f,f[1]=concat(f[1],f[i]));
    [f[1],p]
};

CY_Transmat(sol1,sol2,p)={
    local(A=matid(4),B=matid(4),p1,p2);
    A[,1]=sol1[1]~;
    B[,1]=sol2[1]~;
    for(i=2,4,
        A[,i]=(-z^2)^(sol1[2]==oo)*(deriv(A[,i-1],z)+deriv(A[,i-1],Log)/z);
        B[,i]=(-z^2)^(sol2[2]==oo)*(deriv(B[,i-1],z)+deriv(B[,i-1],Log)/z);
    );
    if(
        sol1[2]==oo,p1=1/p,
        True,p1=p-sol1[2]
    );
    if(
        sol2[2]==oo,p2=1/p,
        True,p2=p-sol2[2]
    );
    A=subst(subst(A,z,p1),Log,log(p1));
    B=subst(subst(B,z,p2),Log,log(p2));
    A*matsolve(B,matid(4))
};



\\ ---------- Methods for the computation of period polynomials ----------

PP_Integrals(f,k)={
		     local(fint=vector(k));
		     fint[1]=truncate(intformal(f/x));
		     for(i=2,k,fint[i]=intformal(fint[i-1]/x));
		     vector(k,X,fint[X]/(2*Pi*I)^X)~
		  };

PP_SL2(g,tau)=(g[1,1]*tau+g[1,2])/(g[2,1]*tau+g[2,2]);

PP_Boundary(P,g,n)=(g[2,1]*tau+g[2,2])^(-n)*subst(P,tau,PP_SL2(g,tau))-P;

PP_PerPol(fi,g)={
		   local(k=#fi,p1,p2,tmp1,tmp2,tau0=(I+g[1,1]*sign(g[2,1]))/max(1,abs(g[2,1])),tau1=PP_SL2(matsolve(g,matid(2)),tau0));
		   
		   p1=(tau-z)^(k-2);
		   p2=subst(bernpol(k-1,tau),tau,tau-z);
                   p2=p2-subst(p2,z,0);
		   p2=PP_Boundary(p2,g,2-k)/(k-1);
		   tmp1=sum(i=0,k-2,polcoef(p1,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*tau0^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau0))))-sum(i=0,k-2,polcoef(p1,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*tau1^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau1))));
		   tmp2=sum(i=0,k-1,polcoef(p2,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*(tau0-1)^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau0))))-sum(i=0,k-1,polcoef(p2,i,z)*sum(j=0,i,(-1)^j*i!/(i-j)!*tau0^(i-j)*subst(fi[j+1],x,exp(2*Pi*I*tau0))));
		   
		   (2*Pi*I)^(k-1)/(k-2)!*(tmp1+tmp2)
		};



\\ ---------- Computations ----------

default(parisize,"1G")
default(realprecision,"100")
default(format,"g.15")

ord = 200;

L = Theta^4-3^6*z*(Theta+1/3)^2*(Theta+2/3)^2;
zStar = -1/(2^3*3^6);
f0 = CY_Solve(L,0,ord,1.0)[1];
f0 *= [(2*Pi*I)^3,0,0,0;0,(2*Pi*I)^2,0,0;0,0,2*Pi*I,0;-144/9*zeta(3),0,0,1]~;
T = matrix(4,4);
T[,1] = f0~;
for(i=1,3,T[,i+1] = z*deriv(T[,i],z)+deriv(T[,i],Log));
T = substvec(T,[z,Log],[zStar,log(-zStar)]);
print("Period matrix:");
printp(T);

ord = 3000;

[f] = mfeigensearch([54,4],[[5,3]]);
f = Ser(mfcoefs(f,ord));
fi = PP_Integrals(f,4);
fPol = PP_PerPol(fi,[23,-3;54,-7]);
wfp = real(subst(fPol,tau,0))/(-5);
wfm = I*imag(subst(fPol,tau,0))/13;
print("\nPeriods of f: ",[wfp,wfm]);
print("Checking period polynomial of f: ",fPol-(wfp*(-5+75*tau-288*tau^2)+wfm*(13-207*tau+756*tau^2)));

[g] = mfeigensearch([54,2],[[5,3]]);
g = Ser(mfcoefs(g,ord));
gi = PP_Integrals(g,2);
gPol = PP_PerPol(gi,[43,-4;54,-5]);
wgp = real(gPol);
wgm = -I*imag(gPol);
print("\nPeriods of g: ",[wgp,wgm]);
print("Checking period polynomial of g: ",gPol-(wgp-wgm));

h = mffrometaquo([9,4;18,-8;27,-12;54,24]);
[F] = mfsearch([54,8],[0,0,0,0,0,0,0,0,0,0,0,0,0,158171/9,10648/9,0,32000/3,54872/9,0,142477/9,-4096/9,0,-566828/9,-42592/9,0,-297485/9,-219488/9,0,-539188/9,13640/9,0,1320661/9,106352/9,0,613940/9,548480/9,0,1394015/9,51392/3,0,-293248/3,148336/9,0,2404846/9,91744/9,0,-121536,-5962616/9,0,301006/3,4376200/9,0,-8529572/9,-3064016/3,0,-8464213/9,24447008/9,0,-12319156/9,-678016/3],1);
F = Ser(mfcoefs(F,ord))/Ser(mfcoefs(h,ord));
Fi = PP_Integrals(F,4);
FPol = PP_PerPol(Fi,[23,-3;54,-7]);
efp = real(subst(FPol,tau,0))/(-5);
efm = I*imag(subst(FPol,tau,0))/13;
print("\nQuasiperiods of f: ",[efp,efm]);
print("Checking period polynomial of F: ",FPol-(efp*(-5+75*tau-288*tau^2)+efm*(13-207*tau+756*tau^2)));

h = mffrometaquo([9,4;18,-8;27,-12;54,24]);
[G] = mfsearch([54,6],[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,8,0,0,4,0,32,-38,0,-6,66],1);
G = Ser(mfcoefs(G,ord))/Ser(mfcoefs(h,ord));
Gi = PP_Integrals(G,2);
GPol = PP_PerPol(Gi,[43,-4;54,-5]);
egp = real(GPol);
egm = -I*imag(GPol);
print("\nQuasiperiods of g: ",[egp,egm]);
print("Checking period polynomial of G: ",GPol-(egp-egm));

print("\nChecking Legendre relations: ",[matdet([wfp,efp;wfm,efm])-(2*Pi*I)^3,matdet([wgp,egp;wgm,egm])-(2*Pi*I)]);

A = [0,-48,-48,0;8,0,0,-24;0,46,-18,0;-3,0,0,-23];
B = [27/4,-9/8,1/8,21/8;0,0,0,-1/3888;0,1/8,1/8,-7/162;0,0,-1/216,5/3888];
print("\nChecking modularity of period matrix:");
printp(T-A*[wfp,efp,0,0;wfm,efm,0,0;0,0,2*Pi*I*wgp,2*Pi*I*egp;0,0,2*Pi*I*wgm,2*Pi*I*egm]*B);
