\\ ---------- Methods for the point counting ----------

\\ S corresponds to the surface s*X*Y*Z-t*(X^3+Y^3+Z^3)=0 and X corresponds to the fiber product with fibers S(s:t) x S(-18*t:s).

S(s,t) = ellinit([-3*s/t,-9*s^2/t^2,(-243*t^3+9*s^3)/t^3,(-729*s*t^3+27*s^4)/t^4,(-19683*t^6+1458*s^3*t^3-27*s^6)/t^6]);

countS(s,t) = {
	 if(t==0,
		3*s.p^poldegree(s.mod),
	 	if(s != 0 && (3*t/s)^3==1,
		     if(#polrootsmod(x^2+3,s)==2,
			3*s.p^poldegree(s.mod),
			2+s.p^poldegree(s.mod)
		     ),
		     S(s,t).no
		)
	 )
};

countX(p,k) = {
	 local(gen,fieldElements,n);
	 gen = ffgen(p^k);
	 fieldElements = List();
	 forvec(X=vector(k,i,[0,p-1]),listput(fieldElements,sum(i=0,k-1,X[i+1]*gen^i)));
	 n = countS(0*gen,1+0*gen)*countS(1+0*gen,0*gen);
	 for(t=1,#fieldElements,n+=countS(1+0*gen,fieldElements[t])*countS(-18*fieldElements[t],1+0*gen));
	 n
};

fixedPointsPhiFrob(p,k) = {
	local(m,gen,xi,fibers,n1,n2,xs,ys,tmp);
	p = p^k;
	m = -1/(2^3*3^6);
	gen = ffgen(if(kronecker(-3,p)==-1,p^2,p^3));
	xi = (polrootsmod(x^2+3,gen)[1]-1)/2;
	fibers = polrootsmod(xi*x^p-x,gen)~;
	fibers = concat([[0,1]],apply(t->[1,t],fibers));
	fibers = apply(v->[v[1]+0*gen,v[2]+0*gen],fibers);
	sum(i=1,#fibers,
		n1 = 0;	
		xs = polrootsmod(xi*x^p-x,gen)~;
		n1 = if(fibers[i][2]==0,1,0);
		for(j=1,#xs,if(substvec(s*x*y*z-t*(x^3+y^3+z^3),[s,t,x,y,z],[fibers[i][1],fibers[i][2],xs[j],1,0])==0,n1++));
		for(j=1,#xs,
			tmp = substvec(s*x*y*z-t*(x^3+y^3+z^3),[s,t,x,z],[fibers[i][1],fibers[i][2],xs[j],1]);
			if(tmp==0,n1+=p,
				tmp = polrootsmod(tmp,gen);
				for(k=1,#tmp,if(tmp[k]^p==tmp[k],n1++));	
			);
		);
		n2 = 0;	
		xs = polrootsmod(1/xi*x^p-x,gen)~;
		n2 = if(fibers[i][1]==0,1,0);
		for(j=1,#xs,if(substvec(t*x*y*z-s*(x^3+y^3+m*z^3),[s,t,x,y,z],[fibers[i][1],fibers[i][2],xs[j],1,0])==0,n2++));
		for(j=1,#xs,
			tmp = substvec(t*x*y*z-s*(x^3+y^3+m*z^3),[s,t,x,z],[fibers[i][1],fibers[i][2],xs[j],1]);
			if(tmp==0,n2+=p,
				tmp = polrootsmod(tmp,gen);
				for(k=1,#tmp,if(tmp[k]^p==tmp[k],n2++));	
			);
		);
		n1*n2
	)
};



\\ ---------- Computations ----------

default(parisize,"1G")

primeList = primes(9)[3..9];

\\ The curves E1 and E are isogenous. 
E1 = S(-6,1);
E = S(0,1);

computeData(p) = {
	my(lFrob,lFrob2,lPhiFrob,lPhiFrob2,trFrobV,trFrob2V,polV,polUmodV);
	lFrob = countX(p,1);
	lFrob2 = countX(p,2);
	lPhiFrob = fixedPointsPhiFrob(p,1);
	lPhiFrob2 = fixedPointsPhiFrob(p,2);
	trFrobV = 1 + (9+2*kronecker(-3,p))*p - 4*p*ellap(E,p) + (9+2*kronecker(-3,p))*p^2 + p^3 - (lFrob+2*lPhiFrob)/3;
	trFrob2V = 1 + (9+2*kronecker(-3,p^2))*p^2 - 4*p^2*(ellap(E,p)^2-2*p) + (9+2*kronecker(-3,p^2))*p^4 + p^6 - (lFrob2+2*lPhiFrob2)/3;
	polV = substvec(1+A*T+p*B*T^2+A*p^3*T^3+p^6*T^4,[A,B],[-trFrobV,(trFrobV^2-trFrob2V)/(2*p)]);
	polV = (f->[f,polV/f])(factor(polV)[1,1]);
	polV = apply(f->f/subst(f,T,0),polV);
	trFrobUmodV = 1 + (13+6*kronecker(-3,p))*p - 16*p*ellap(E,p) + (13+6*kronecker(-3,p))*p^2 + p^3 - lFrob - trFrobV;
	trFrob2UmodV = 1 + (13+6*kronecker(-3,p^2))*p^2 - 16*p^2*(ellap(E,p)^2-2*p) + (13+6*kronecker(-3,p^2))*p^4 + p^6 - lFrob2 - trFrob2V;
	polUmodV = substvec(1+A*T+p*B*T^2+A*p^3*T^3+p^6*T^4,[A,B],[-trFrobUmodV,(trFrobUmodV^2-trFrob2UmodV)/(2*p)]);
	polUmodV = (f->[f,polUmodV/f])(factor(polUmodV)[1,1]);
	polUmodV = apply(f->f/subst(f,T,0),polUmodV);
	[p,lFrob,lPhiFrob,lFrob2,lPhiFrob2,[polV[1],polV[2]],polUmodV,ellap(E,p)]
};

print("Computing [p, L(F_p), L(phi*F_p), L(F_p^2), L(phi*F_p^2), factors of det(1-T*F_p|V), factors of det(1-T*F_p|U/V), tr(F_p|H^1(E))]:");
apply(p->print(computeData(p)),primeList);
