\\ ---------- Methods for the computation of periods ----------

CY_Indicials(L,p)={
    L=polcoef(CY_Pointchange(L,p),0,z);
    local(indicials=factor(L));
    Set(vector(matsize(indicials)[1],X,[-polcoef(indicials[X,1],0,Theta)/polcoef(indicials[X,1],1,Theta),indicials[X,2]]))
};

CY_Pointchange(L,p)={
    local(tmp=1,Lnew=0);
    if(
        p!=oo,for(k=0,4,Lnew+=subst(polcoef(L,k,Theta),z,z+p)*tmp;tmp=(z+p)/z*(z*deriv(tmp,z)+tmp*Theta)),
        p==oo,Lnew=subst(subst(L,Theta,-Theta),z,1/z)
    );
    Lnew/z^(-poldegree(subst(lift(Lnew),z,1/z),z))
};

CY_Solve(L,p,ord,numval)={
    local(ind=CY_Indicials(L,p),indset=Set(apply(x->x[1],ind)));
    L=CY_Pointchange(L,p);
    local(f=vector(#ind,X,vector(ind[X][2],Y,0)));
    local(disc,num,c=0);
    for(X=1,#f,
        f[X][1]=z^ind[X][1]*(1+numval-numval);
        for(k=ind[X][1]+1,ord,
            for(Y=1,ind[X][2],
                disc=subst(polcoef(L,0,z),Theta,k);
                if(setsearch(indset,k)&&Y<ind[X][2],disc=subst(deriv(polcoef(L,0,z),Theta),Theta,k);c=1);
                if(setsearch(indset,k)==0||c==1,
                    num=sum(i=0,min(poldegree(L,z),k),sum(j=1,Y,subst(diffop(polcoef(L,i,z),[Theta],[1],j-1+c),Theta,k-i)/(j-1)!*polcoef(f[X][Y+1-j],k-i,z)));
                    f[X][Y]+=-num/disc*z^k
                );
                c=0;
            )
        )
    );
    f=apply(x->vector(#x,Y,sum(k=0,Y-1,x[Y-k]/k!*Log^k)),f);
    for(i=2,#f,f[1]=concat(f[1],f[i]));
    [f[1],p]
};

CY_Transmat(sol1,sol2,p)={
    local(A=matid(4),B=matid(4),p1,p2);
    A[,1]=sol1[1]~;
    B[,1]=sol2[1]~;
    for(i=2,4,
        A[,i]=(-z^2)^(sol1[2]==oo)*(deriv(A[,i-1],z)+deriv(A[,i-1],Log)/z);
        B[,i]=(-z^2)^(sol2[2]==oo)*(deriv(B[,i-1],z)+deriv(B[,i-1],Log)/z);
    );
    if(
        sol1[2]==oo,p1=1/p,
        True,p1=p-sol1[2]
    );
    if(
        sol2[2]==oo,p2=1/p,
        True,p2=p-sol2[2]
    );
    A=subst(subst(A,z,p1),Log,log(p1));
    B=subst(subst(B,z,p2),Log,log(p2));
    A*matsolve(B,matid(4))
};



\\ ---------- Computations ----------

default(parisize,"1G")
default(realprecision,"200")
default(format,"g.15")

ord = 1500;

L = (4294967296*z^8+1409286144*z^7+94371840*z^6+5505024*z^5+606208*z^4-21504*z^3+1440*z^2-84*z+1)*Theta^4+(17179869184*z^8+2818572288*z^7+201326592*z^6+55050240*z^5+1212416*z^4+129024*z^3+2688*z^2-168*z)*Theta^3+(25769803776*z^8+2281701376*z^7+283115520*z^6+30932992*z^5+819200*z^4+395264*z^3+3744*z^2-136*z)*Theta^2+(17179869184*z^8+872415232*z^7+201326592*z^6-4980736*z^5+212992*z^4+297984*z^3+2112*z^2-52*z)*Theta+(4294967296*z^8+134217728*z^7+50331648*z^6-6291456*z^5+98304*z^4+77824*z^3+384*z^2-8*z);
zStar = I/2^4;
f0 = CY_Solve(L,0,ord,1.0);
f0[1] *= [(2*Pi*I)^3,0,0,0;0,(2*Pi*I)^2,0,0;0,0,2*Pi*I,0;1/4*zeta(3),0,0,1]~;
f1 = CY_Solve(L,zStar,ord,1.0);
transmat = CY_Transmat(f0,f1,zStar/5);
T = matrix(4,4);
T[,1] = f1[1]~;
T[,2] = (z+zStar)*deriv(T[,1],z);
T[,3] = (z+zStar)*deriv(T[,2],z);
T[,3] = (z+zStar)*deriv(T[,3],z);
T[,4] = (z+zStar)*deriv(T[,3],z);
T = transmat*subst(T,z,0);
A = [48,0,48,0;0,48,0,48;-9,16,-1,16;-2,1,-2,9];
B = [1, -1/2, -7/170 + 3/85*I, -71/170 - 6/85*I; 0, 0, 0, 1; 0, 1/2, 7/170 - 3/85*I, 71/170 + 6/85*I; 0, 0, -7/85 + 6/85*I, 14/85 - 12/85*I];
B = [14+12*I,-7-6*I,-1,0;0,0,0,1;0,1,0,1;0,0,1,-2];
print("Checking splitting of period matrix:");
printp(A^(-1)*T*B^(-1));

[wp1,ep1,wp2,ep2] = concat(Vec((A^(-1)*T*B^(-1))[3..4,3..4]~))~;
[wE1,eE1] = intnum(x=I,-(1+sqrt(1-4*I))/2,[1,x]/sqrt(x^3+(1-I)*x^2+1));
[wE2,eE2] = intnum(x=I,-(1-sqrt(1-4*I))/2,[1,x]/sqrt(x^3+(1-I)*x^2+1));
print("Checking identity involving periods of elliptic curve:");
printp([wp1,ep1;wp2,ep2]-2*Pi*I*[4,0;1,1]*[wE1,eE1;wE2,eE2]*[1/48,4111/346800-499/173400*I;0,-13/14450+42/7225*I]);
